/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import dk.sundhedsdatastyrelsen.skr._2024._01._01.GetPersonalDataCardRequest;
import dk.sundhedsdatastyrelsen.skr._2024._01._01.GetPersonalDataCardResponse;
import org.hl7.v3.II;

import dk.stamkort.OID;
import dk.stamkort.util.RequestContextVO;

public class PersonalDataCardRegisterClient extends ReusingDGWSClient<GetPersonalDataCardRequest, GetPersonalDataCardResponse> {

    public PersonalDataCardRegisterClient(String endpoint, int connectTimeoutMillis, int readTimeoutMillis) throws JAXBException {
        super("SKR", GetPersonalDataCardRequest.class, GetPersonalDataCardResponse.class,
              endpoint, connectTimeoutMillis, readTimeoutMillis);
    }

    public GetPersonalDataCardResponse call(String cpr, RequestContextVO requestContext) throws DGWSClientException {
        GetPersonalDataCardRequest request = new GetPersonalDataCardRequest();
        II id = new II();
        id.setRoot(OID.CPR);
        id.setAssigningAuthorityName("CPR");
        id.setExtension(cpr);
        request.setId(id);

        return super.call(request, requestContext);
    }

    @Override
    protected QName getRequestQName() {
        return new QName("http://sundhedsdatastyrelsen.dk/skr/2024/01/01", "GetPersonalDataCardRequest");
    }

    @Override
    protected String getSOAPAction() { return "http://sundhedsdatastyrelsen.dk/skr/2024/01/01#GetPersonalDataCard"; }
}
