/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import dk.stamkort.domain.Dentist;
import dk.stamkort.domain.GeneralPractitioner;
import dk.stamkort.domain.YderVO;
import dk.stamkort.util.YderNrUtil;
import oio.medcom.cprservice._1_0.GetPersonWithHealthCareInformationOut;
import org.hl7.v3.POCDMT000040ClinicalDocument;

import java.util.*;

import static java.util.Optional.*;

public class RegisterData {
    private String cprnr;
    private GetPersonWithHealthCareInformationOut scesData;
    private List<ChildCustodyCustodyData> childCustodyCustodyData = new ArrayList<>();
    private List<CustodyByCustodyData> custodyByCustodyData = new ArrayList<>();
    private POCDMT000040ClinicalDocument clinicalDocument;
    private Boolean hasLivingWill;
    private Boolean hasTreatmentWill;
    private Boolean hasOrganDonorRegistration;
    private final Map<String, YderVO> yderInfo = new HashMap<>();
    private Optional<Boolean> hasNoResuscitation = empty();
    private Optional<GeneralPractitioner> generalPractitioner = empty();
    private Optional<Dentist> dentist = empty();

    public RegisterData(String cprnr) { this.cprnr = cprnr; }

    public String getCprnr() {
        return cprnr;
    }

    public void setCprnr(String cprnr) {
        this.cprnr = cprnr;
    }

    public GetPersonWithHealthCareInformationOut getScesData() {
        return scesData;
    }

    public void setScesData(GetPersonWithHealthCareInformationOut scesData) {
        this.scesData = scesData;
    }

    public List<ChildCustodyCustodyData> getChildCustodyCustodyData() {
        return childCustodyCustodyData;
    }

    public void addChildCustodyCustodyData(ChildCustodyCustodyData childCustodyCustodyData) {
        this.childCustodyCustodyData.add(childCustodyCustodyData);
    }

    public List<CustodyByCustodyData> getCustodyByCustodyData() {
        return custodyByCustodyData;
    }

    public void addCustodyByCustodyData(CustodyByCustodyData custodyByCustodyData) {
        this.custodyByCustodyData.add(custodyByCustodyData);
    }

    public POCDMT000040ClinicalDocument getClinicalDocument() {
        return clinicalDocument;
    }

    public void setClinicalDocument(POCDMT000040ClinicalDocument d) {
        this.clinicalDocument = d;
    }

    public Boolean getHasLivingWill() {
        return hasLivingWill;
    }

    public void setHasLivingWill(Boolean hasLivingWill) {
        this.hasLivingWill = hasLivingWill;
    }

    public Boolean getHasTreatmentWill() {
        return hasTreatmentWill;
    }

    public void setHasTreatmentWill(Boolean hasTreatmentWill) {
        this.hasTreatmentWill = hasTreatmentWill;
    }

    public Boolean getHasOrganDonorRegistration() {
        return hasOrganDonorRegistration;
    }

    public void setHasOrganDonorRegistration(Boolean hasOrganDonorRegistration) {
        this.hasOrganDonorRegistration = hasOrganDonorRegistration;
    }

    /** @deprecated Only exists for backwards compatibility with CDA V2.
     * Replaced by GeneralPractitioner and Dentist types */
    @Deprecated
    public YderVO getYder(String ydernr) {
        return yderInfo.get(YderNrUtil.padZero(ydernr));
    }

    /** @deprecated Only exists for backwards compatibility with CDA V2.
     * Replaced by GeneralPractitioner and Dentist types */
    @Deprecated
    public void addYder(String yderNr, YderVO yder) {
        if (yderNr != null && yder != null) {
            yderInfo.put(YderNrUtil.padZero(yderNr), yder);
        }
    }

    public void setHasNoResuscitation(Boolean hasNoResuscitation) { setHasNoResuscitation(of(hasNoResuscitation)); }

    public void setHasNoResuscitation(Optional<Boolean> hasNoResuscitation) {
        this.hasNoResuscitation = hasNoResuscitation;
    }

    public Optional<Boolean> getHasNoResuscitation() { return hasNoResuscitation; }

    public void setGeneralPractitioner(GeneralPractitioner gp) { this.generalPractitioner = ofNullable(gp); }
    public Optional<GeneralPractitioner> getGeneralPractitioner() { return generalPractitioner; }

    public void setDentist(Dentist dentist) { this.dentist = ofNullable(dentist); }
    public Optional<Dentist> getDentist() { return dentist; }
}