/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import dk.nsi.__.stamdata._3.ObjectFactory;
import dk.nsi.__.stamdata._3.YderRequestType;
import dk.nsi.__.stamdata._3.YderResponseType;
import dk.stamkort.domain.YderNr;
import dk.stamkort.domain.YderVO;
import dk.stamkort.util.RequestContextVO;

public class YderClient extends ReusingDGWSClient<YderRequestType, YderResponseType> {


    public YderClient(String endpoint, int connectTimeoutMillis, int readTimeoutMillis) throws JAXBException {
        super("SYES", YderRequestType.class, YderResponseType.class, endpoint, connectTimeoutMillis, readTimeoutMillis);
    }


    public YderVO call(String yderNr,RequestContextVO requestContext) throws DGWSClientException {
    	if (yderNr != null && yderNr.length() > 1) {
            YderRequestType yderRequestType = new YderRequestType();
            yderRequestType.setYdernummer(yderNr);

            JAXBElement<YderRequestType> request = new ObjectFactory().createYderRequestStructure(yderRequestType);

            YderResponseType yderResponse = super.call(request, requestContext);
            if (yderResponse.getYder() == null) { return null;}

            YderVO yder = new YderVO();

            yder.setYdernr(new YderNr(yderNr));
            yder.setOrganizationName(yderResponse.getYder().getPraksisbetegnelse());
            yder.setAddress(yderResponse.getYder().getAdresse());
            yder.setPostcode(yderResponse.getYder().getPostnummer());
            yder.setCity(yderResponse.getYder().getPostdistrikt());
            yder.setPhone(yderResponse.getYder().getTelefonnummer());
            yder.setEmail(yderResponse.getYder().getEmail());

            return yder;
    	}
    	return null;
    }

    @Override
    protected QName getRequestQName() {
        return new QName("http://nsi.dk/-/stamdata/3.0", "YderRequestType");
    }

    @Override
    protected String getSOAPAction() {
        return "http://nsi.dk/sdm/Gateway";
    }
}
