/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

import java.util.HashMap;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

public class DataSourceHealthIndicator implements HealthIndicator {

	private static final Logger LOGGER = Logger.getLogger(DataSourceHealthIndicator.class);
	
	private String dbName;
	
	private NamedParameterJdbcTemplate toCheck;
	private boolean dbOk = true;
	private String error = "";
	
	public DataSourceHealthIndicator(String dbName, DataSource ds) {
		this.toCheck = new NamedParameterJdbcTemplate(ds);
		this.dbName = dbName;
	}
	
	@Override
	public boolean isHealthy() {
		LOGGER.debug("Kalder health på databasen: "+dbName);
		try {
			toCheck.queryForObject("SELECT 1", new HashMap<String, String>(), Integer.class);
			dbOk = true;
		} catch (Exception e) {
			LOGGER.error(e);
			dbOk = false;
			error = e.getMessage();
		}
		return dbOk;
	}

	@Override
	public int getStatusCode() {
		return (isHealthy() ? 200 : 500);
	}

	@Override
	public String getName() {
		return dbName+" database";
	}

	@Override
	public String getStatusText() {
		return (dbOk ? "Forbindelse til database "+dbName+" ok" : "Forbindelse til database "+dbName+" fejler: "+error);
	}
}
