/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

/**
 * Created by obj on 02-11-2017.
 */
public enum FaultCode {
    GENERAL_ERROR(100, "Der opstod en fejl", false),

    NO_SCES_DATA_FOUND(102, "Ingen data fundet", true),

    CALL_PERSONAL_DATA_CARD_REGISTER_ERROR(103, "Der opstod en fejl under kald af stamkort registret", false),
    CALL_SCES_ERROR(104, "Der opstod en fejl under kald af CPR enkeltopslagsservice", false),
    CALL_LIVING_WILL_ERROR(105, "Der opstod en fejl under kald af livstestamenteregistret", false),
    CALL_ORGAN_DONOR_REGISTER_ERROR(106, "Der opstod en fejl under kald af organdonorregistret", false),
    CALL_TREATMENT_WILL_ERROR(107, "Der opstod en fejl under kald af behandlingstestamenteregistret", false),
    CALL_SCES_CUSTODY_ERROR(108, "Der opstod en fejl under opslag af pårørende til person i CPR enkeltopslagsservice", false),
    CALL_SCES_STATUS_30(109, "Det er ikke muligt at hente stamkortet, da CPR-nummeret er inaktivt", true),
    CALL_SCES_STATUS_50(110, "Det er ikke muligt at hente stamkortet, da CPR-nummeret er inaktivt", true),
    CALL_SCES_STATUS_60(111, "Det er ikke muligt at hente stamkortet, da CPR-nummeret er inaktivt", true),
    CALL_SCES_STATUS_90(112, "Det er ikke muligt at hente stamkortet, da borgeren er afgået ved døden for mere end 1 år siden", true),
    CALL_FGVHR_ERROR(113, "Der opstod en fejl under kald af FGVHR", false),

    MAPPING_ERROR_MISSING_DATA(201, "Manglende data i datakilde for stamkortet", false),
    MAPPING_ERROR_INVALID_DATA(202, "Ugyldig data i datakilde for stamkortet", false),

    AUTHORIZATION_ERROR_INVALID_HEADER(500, "Sikkerhedsbillet er ikke gyldig", true), 
    AUTHORIZATION_ERROR_NO_DELEGATION(500, "Delegering ikke tilladt", true), 
    AUTHORIZATION_ERROR_CITIZEN_NOT_ALLOWED(500, "Borgerbillet ikke tilladt", true),

    AUTHORIZATION_ERROR_UNKNOWN_USER_TYPE (500, "Ukendt brugertype", true);

    private int code;
    private String message;
    private boolean warningOnly;

    public int getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public boolean isWarningOnly() {
        return warningOnly;
    }

    FaultCode(int code, String message, boolean warningOnly) {
        this.code = code;
        this.message = message;
        this.warningOnly = warningOnly;
    }

    @Override
    public String toString() {
        return code + ": " + message;
    }
}
