/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;

@Component
public class Properties {
    private static final Logger log = LogManager.getLogger(Properties.class);
    private ZonedDateTime btrIntegrationStartDate;

    @Value("${author.institution.root:1.2.208.176.1.1}")
    private String authorInstitutionRoot;

    @Value("${author.institution.extension:1126211000016009}")
    private String authorInstitutionExtension;

    @Value("${author.institution.assigningAuthorityName:SOR}")
    private String authorInstitutionAssigningAuthorityName;

    @Value("${author.institution.name:Sundhedsdatastyrelsen}")
    private String authorInstitutionName;

    @Value("${whitelisted.level3.cvrs:}")
    private String whitelistedLevel3CVRs;

    @Value("${health.certificate-expires-warning}")
    private int healthCertificateExpiresWarning;

    @Value("${isProduction:false}")
    private boolean isProduction;

    @Value("${ignoreInvalidIdcardInTestMode:true}")
    private boolean ignoreInvalidIdcardInTestMode;

    @Value("${sces.enable:true}")
    private boolean scesIntegrationEnabled;

    @Value("${sces.endpoint}")
    private String scesEndpoint;

    @Value("${sces.connect.timeout.millis:5000}")
    private int scesConnectTimeoutMillis;

    @Value("${sces.read.timeout.millis:10000}")
    private int scesReadTimeoutMillis;

    @Value("${sts.endpoint}")
    private String stsEndpoint;

    @Value("${odr.enable:true}")
    private boolean odrIntegrationEnabled;

    @Value("${odr.endpoint}")
    private String odrEndpoint;

    @Value("${odr.connect.timeout.millis:5000}")
    private int odrConnectTimeoutMillis;

    @Value("${odr.read.timeout.millis:10000}")
    private int odrReadTimeoutMillis;

    @Value("${ltr.enable:true}")
    private boolean ltrIntegrationEnabled;

    @Value("${ltr.endpoint}")
    private String ltrEndpoint;

    @Value("${ltr.connect.timeout.millis:5000}")
    private int ltrConnectTimeoutMillis;

    @Value("${ltr.read.timeout.millis:10000}")
    private int ltrReadTimeoutMillis;

    @Value("${btr.startdatetime:2019-01-01 00:00:00}")
    private String btrIntegrationStartDateTimeString;

    @Value("${btr.enable:true}")
    private boolean btrIntegrationEnabled;

    @Value("${btr.endpoint}")
    private String btrEndpoint;

    @Value("${btr.connect.timeout.millis:5000}")
    private int btrConnectTimeoutMillis;

    @Value("${btr.read.timeout.millis:10000}")
    private int btrReadTimeoutMillis;

    @Value("${skr.enable:true}")
    private boolean skrIntegrationEnabled;

    @Value("${skr.endpoint}")
    private String skrEndpoint;

    @Value("${skr.connect.timeout.millis:5000}")
    private int skrConnectTimeoutMillis;

    @Value("${skr.read.timeout.millis:10000}")
    private int skrReadTimeoutMillis;

    @Value("${syes.endpoint}")
    private String syesEndpoint;

    @Value("${syes.connect.timeout.millis:5000}")
    private int syesConnectTimeoutMillis;

    @Value("${syes.read.timeout.millis:10000}")
    private int syesReadTimeoutMillis;

    @Value("${dds.repository.unique.id}")
    private String ddsRepositoryUniqueId;

    @Value("${dds.home.community.id}")
    private String ddsHomeCommunityId;

    @Value("${datasource-fsk.jndi-name:}")
    private String dataSourceFSKJNDIName;

    @Value("${datasource-stm.jndi-name:}")
    private String dataSourceStamdataJNDIName;

    @Value("${minlog.read-activity.text:Opslag i Det Fælles Stamkort}")
    private String minLogReadActivityText;

    @Value("${minlog.on.idcard.level3.enabled}")
    private String minLogOnIdcardLevel3Enabled;

    @Value("${dgwsclient.pool.size:50}")
    private Integer dgwsClientPoolSize;

    @Value("${dgwsclient.timeout.millis:10000}")
    private Long dgwsClientTimeoutMillis;

    @Value("${call.sces.status.30:}")
    private String callScesStatus30;

    @Value("${call.sces.status.50:}")
    private String callScesStatus50;

    @Value("${call.sces.status.60:}")
    private String callScesStatus60;

    @Value("${call.sces.status.90:}")
    private String callScesStatus90;

    @Value("${fgvhr.endpoint}")
    private String fgvhrEndpoint;

    @Value("${fgvhr.enable:true}")
    private boolean fgvhrIntegrationEnabled;

    @Value("${fgvhr.connect.timeout.millis:2000}")
    private int fgvhrConnectTimeoutMillis;

    @Value("${fgvhr.read.timeout.millis:7000}")
    private int fgvhrReadTimeoutMillis;

    @Value("${fgvhr.sts.keystore}")
    private String fgvhrStsKeystore;

    @Value("${fgvhr.sts.keystore.password}")
    private String fgvhrStsKeystorePassword;

    @Value("${fgvhr.idcard.subject.name}")
    private String fgvhrIdcardSubjectName;

    @Value("${fgvhr.idcard.subject.id}")
    private String fgvhrIdcardSubjectId;

    @Value("${fgvhr.idcard.system.name}")
    private String fgvhrIdcardSystemName;

    public String getCallScesStatus30() {
		return callScesStatus30;
	}

    public String getCallScesStatus50() {
		return callScesStatus50;
	}

    public String getCallScesStatus60() {
		return callScesStatus60;
	}

    public String getCallScesStatus90() {
		return callScesStatus90;
	}

	@Override
    public String toString() {
        return "Properties{" +
                "authorInstitutionRoot='" + authorInstitutionRoot + '\'' +
                ", authorInstitutionExtension='" + authorInstitutionExtension + '\'' +
                ", authorInstitutionAssigningAuthorityName='" + authorInstitutionAssigningAuthorityName + '\'' +
                ", authorInstitutionName='" + authorInstitutionName + '\'' +
                ", btrIntegrationStartDate=" + btrIntegrationStartDate +
                ", whitelistedLevel3CVRs='" + whitelistedLevel3CVRs + '\'' +
                ", healthCertificateExpiresWarning='" + healthCertificateExpiresWarning + '\'' +
                ", isProduction=" + isProduction +
                ", ignoreInvalidIdcardInTestMode=" + ignoreInvalidIdcardInTestMode +
                ", scesIntegrationEnabled=" + scesIntegrationEnabled +
                ", scesEndpoint='" + scesEndpoint + '\'' +
                ", scesConnectTimeoutMillis=" + scesConnectTimeoutMillis +
                ", scesReadTimeoutMillis=" + scesReadTimeoutMillis +
                ", stsEndpoint='" + stsEndpoint + '\'' +
                ", odrIntegrationEnabled=" + odrIntegrationEnabled +
                ", odrEndpoint='" + odrEndpoint + '\'' +
                ", odrConnectTimeoutMillis=" + odrConnectTimeoutMillis +
                ", odrReadTimeoutMillis=" + odrReadTimeoutMillis +
                ", ltrIntegrationEnabled=" + ltrIntegrationEnabled +
                ", ltrEndpoint='" + ltrEndpoint + '\'' +
                ", ltrConnectTimeoutMillis=" + ltrConnectTimeoutMillis +
                ", ltrReadTimeoutMillis=" + ltrReadTimeoutMillis +
                ", btrIntegrationStartDateTimeString='" + btrIntegrationStartDateTimeString + '\'' +
                ", btrIntegrationEnabled=" + btrIntegrationEnabled +
                ", btrEndpoint='" + btrEndpoint + '\'' +
                ", btrConnectTimeoutMillis=" + btrConnectTimeoutMillis +
                ", btrReadTimeoutMillis=" + btrReadTimeoutMillis +
                ", skrIntegrationEnabled=" + skrIntegrationEnabled +
                ", skrEndpoint='" + skrEndpoint + '\'' +
                ", skrConnectTimeoutMillis=" + skrConnectTimeoutMillis +
                ", skrReadTimeoutMillis=" + skrReadTimeoutMillis +
                ", syesEndpoint='" + syesEndpoint + '\'' +
                ", syesConnectTimeoutMillis=" + syesConnectTimeoutMillis +
                ", syesReadTimeoutMillis=" + syesReadTimeoutMillis +
                ", ddsRepositoryUniqueId='" + ddsRepositoryUniqueId + '\'' +
                ", ddsHomeCommunityId='" + ddsHomeCommunityId + '\'' +
                ", primaryDatasourceJNDIName='" + dataSourceFSKJNDIName + '\'' +
                ", stamdataDatasourceJNDIName='" + dataSourceStamdataJNDIName + '\'' +
                ", minLogReadActivityText='" + minLogReadActivityText + '\'' +
                ", minLogOnIdcardLevel3Enabled='" + minLogOnIdcardLevel3Enabled + '\'' +
                ", dgwsClientPoolSize=" + dgwsClientPoolSize +
                ", dgwsClientTimeoutMillis=" + dgwsClientTimeoutMillis +
                ", callScesStatus30=" +  getCallScesStatus30() +
                ", callScesStatus50=" +  getCallScesStatus50() +
                ", callScesStatus60=" +  getCallScesStatus60() +
                ", callScesStatus90=" +  getCallScesStatus90() +
                ", fgvhrEndpoint=" +  fgvhrEndpoint +
                ", fgvhrIntegrationEnabled=" +  fgvhrIntegrationEnabled +
                ", fgvhrConnectTimeoutMillis=" +  fgvhrConnectTimeoutMillis +
                ", fgvhrReadTimeoutMillis=" +  fgvhrReadTimeoutMillis +
                ", fgvhrStsKeystore=" +  fgvhrStsKeystore +
                ", fgvhrIdcardSubjectName=" +  fgvhrIdcardSubjectName +
                ", fgvhrIdcardSubjectId=" +  fgvhrIdcardSubjectId +
                '}';
    }

    @PostConstruct
    public void init() throws ParseException {
        
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        LocalDateTime ldt = LocalDateTime.parse(btrIntegrationStartDateTimeString, dateTimeFormatter);
        btrIntegrationStartDate = ldt.atZone(ZoneId.systemDefault());

        log.info(toString());
    }

    public String getAuthorInstitutionRoot() {
        return authorInstitutionRoot;
    }

    public String getAuthorInstitutionExtension() {
        return authorInstitutionExtension;
    }

    public String getAuthorInstitutionAssigningAuthorityName() {
        return authorInstitutionAssigningAuthorityName;
    }

    public String getAuthorInstitutionName() {
        return authorInstitutionName;
    }

    public String getWhitelistedLevel3CVRs() {
        return whitelistedLevel3CVRs;
    }

    public int getHealthCertificateExpiresWarning() {
        return healthCertificateExpiresWarning;
    }

    public boolean isProduction() {
        return isProduction;
    }

    public boolean isIgnoreInvalidIdcardInTestMode() {
        return ignoreInvalidIdcardInTestMode;
    }

    public boolean isScesIntegrationEnabled() {
        return scesIntegrationEnabled;
    }

    /** The SCES endpoint to use */
    public String getScesEndpoint() {
        return scesEndpoint;
    }

    /** The SCES connect timeout */
    public int getScesConnectTimeoutMillis() {
        return scesConnectTimeoutMillis;
    }

    /** The SCES read timeout */
    public int getScesReadTimeoutMillis() {
        return scesReadTimeoutMillis;
    }

    public String getStsEndpoint() {
        return stsEndpoint;
    }

    public boolean isOdrIntegrationEnabled() {
        return odrIntegrationEnabled;
    }

    public String getOdrEndpoint() {
        return odrEndpoint;
    }

    public int getOdrConnectTimeoutMillis() {
        return odrConnectTimeoutMillis;
    }

    public int getOdrReadTimeoutMillis() {
        return odrReadTimeoutMillis;
    }

    public boolean isLtrIntegrationEnabled() {
        return ltrIntegrationEnabled;
    }

    public String getLtrEndpoint() {
        return ltrEndpoint;
    }

    public int getLtrConnectTimeoutMillis() {
        return ltrConnectTimeoutMillis;
    }

    public int getLtrReadTimeoutMillis() {
        return ltrReadTimeoutMillis;
    }

    public ZonedDateTime getBtrIntegrationStartDateTime() {
        return btrIntegrationStartDate;
    }

    public boolean isBtrIntegrationEnabled() {
        return btrIntegrationEnabled;
    }

    public String getBtrEndpoint() {
        return btrEndpoint;
    }

    public int getBtrConnectTimeoutMillis() {
        return btrConnectTimeoutMillis;
    }

    public int getBtrReadTimeoutMillis() {
        return btrReadTimeoutMillis;
    }

    public boolean isSkrIntegrationEnabled() {
        return skrIntegrationEnabled;
    }

    public String getSkrEndpoint() {
        return skrEndpoint;
    }

    public int getSkrConnectTimeoutMillis() {
        return skrConnectTimeoutMillis;
    }

    public int getSkrReadTimeoutMillis() {
        return skrReadTimeoutMillis;
    }

    public String getSyesEndpoint() {  return syesEndpoint; }

    public int getSyesConnectTimeoutMillis() {
        return syesConnectTimeoutMillis;
    }

    public int getSyesReadTimeoutMillis() {
        return syesReadTimeoutMillis;
    }

    /** DDS RepositoryUniqueId */
    public String getDdsRepositoryUniqueId() {
        return ddsRepositoryUniqueId;
    }

    /** DDS home community id */
    public String getDdsHomeCommunityId() {
        return ddsHomeCommunityId;
    }

    public String getDataSourceFSKJNDIName() {
        return dataSourceFSKJNDIName;
    }

    public String getDataSourceStamdataJNDIName() {
        return dataSourceStamdataJNDIName;
    }

    public String getMinLogReadActivityText() {
        return minLogReadActivityText;
    }

    public String getMinLogOnIdcardLevel3Enabled() {
        return minLogOnIdcardLevel3Enabled;
    }

    public Integer getDgwsClientPoolSize() {
        return dgwsClientPoolSize;
    }

    public Long getDgwsClientTimeoutMillis() {
        return dgwsClientTimeoutMillis;
    }

    public AuthorProperties createAuthorProperties() {
        return new AuthorProperties(authorInstitutionRoot, authorInstitutionExtension, authorInstitutionAssigningAuthorityName, authorInstitutionName );
    }

    public String getFgvhrEndpoint() {
        return fgvhrEndpoint;
    }

    public boolean isFgvhrIntegrationEnabled() {
        return fgvhrIntegrationEnabled;
    }

    public int getFgvhrConnectTimeoutMillis() {
        return fgvhrConnectTimeoutMillis;
    }

    public int getFgvhrReadTimeoutMillis() {
        return fgvhrReadTimeoutMillis;
    }

    public int getFgvhrStsConnectTimeoutMillis() {
        return fgvhrConnectTimeoutMillis;
    }

    public int getFgvhrStsReadTimeoutMillis() {
        return fgvhrReadTimeoutMillis;
    }

    public String getFgvhrStsKeystore() {
        return fgvhrStsKeystore;
    }

    public String getFgvhrStsKeystorePassword() {
        return fgvhrStsKeystorePassword;
    }

    public String getFgvhrIdcardSubjectName() {
        return fgvhrIdcardSubjectName;
    }

    public String getFgvhrIdcardSubjectId() {
        return fgvhrIdcardSubjectId;
    }

    public String getFgvhrIdcardSystemName() {
        return fgvhrIdcardSystemName;
    }

}
