/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

import dk.nsi.hsuid.HealthcareServiceUserIdentificationHeaderUtil;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import dk.nsi.hsuid.hsuidattributes.ValidatedHsuidAttributes;
import dk.sds.nsp.security.SecurityContext;
import dk.sdsd.nsp.slalog.api.SLALogItem;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;


public class RequestContextVO {

	private SLALogItem slaLogItem;

	private String patientInContext;

	private RequestContextEnricher requestContextEnricher;

	private HsuidHeader hsuidHeader;

	public enum UserType {
		Citizen,
		HealthCareProfessional,
		SystemUser
	}

	private UserType userType;
	private String givenName;
	private String surName;
	private String authorizationCode;
	private String educationCode;
	private String messageId;
	private String conversationid;
	private String userCprIdentifier;
	private String itSystemName;
	private SecurityContext.Organisation organisation;

	public RequestContextVO() { }

	private static JAXBContext jaxbContext;

	static {
		try {
			jaxbContext = JAXBContext.newInstance(HsuidHeader.class);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
	}

	protected HsuidHeader getHsuidHeader() {
		return hsuidHeader;
	}

	public SLALogItem getSlaLogItem() {
		return slaLogItem;
	}

	public void setSlaLogItem(SLALogItem slaLogItem) {
		this.slaLogItem = slaLogItem;
	}

	public void setPatientInContext(HsuidHeader hsuidHeader) {
		if(hsuidHeader != null) {
			ValidatedHsuidAttributes validatedHsuidAttributes = HealthcareServiceUserIdentificationHeaderUtil.getValidatedHsuidAttributes(hsuidHeader);
			if (validatedHsuidAttributes.isCitizen()) {
				this.patientInContext = validatedHsuidAttributes.getValidatedCitizenHsuidAttributes().getCitizenCivilRegistrationNumber();
			}
			if (validatedHsuidAttributes.isHealthCareProfessional()) {
				this.patientInContext = validatedHsuidAttributes.getValidatedHealthcareProfessionalHsuidAttributes().getCitizenCivilRegistrationNumber();
			} else if (validatedHsuidAttributes.isUnauthorizedProfessionl()) {
				this.patientInContext = validatedHsuidAttributes.getValidatedUnauthorizedProfesionalHsuidAttributes().getCitizenCivilRegistrationNumber();
			}
			this.hsuidHeader = hsuidHeader;
		}
	}

	public String getPatientInContext() {
		return patientInContext;
	}


	public RequestContextEnricher getRequestContextEnricher() {
		return requestContextEnricher;
	}


	public void setRequestContextEnricher(RequestContextEnricher requestContextEnricher) {
		this.requestContextEnricher = requestContextEnricher;
	}

	public UserType getUserType() {
		return userType;
	}

	public void setUserType(UserType userType) {
		this.userType = userType;
	}

	public String getGivenName() {
		return givenName;
	}

	public void setGivenName(String givenName) {
		this.givenName = givenName;
	}

	public String getSurName() {
		return surName;
	}

	public void setSurName(String surName) {
		this.surName = surName;
	}

	public String getAuthorizationCode() {
		return authorizationCode;
	}

	public void setAuthorizationCode(String authorizationCode) {
		this.authorizationCode = authorizationCode;
	}

	public String getEducationCode() {
		return educationCode;
	}

	public void setEducationCode(String educationCode) {
		this.educationCode = educationCode;
	}

	public String getMessageId() {
		return messageId;
	}

	public void setMessageId(String messageId) {
		this.messageId = messageId;
	}

	public String getConversationid() {
		return conversationid;
	}

	public void setConversationid(String conversationid) {
		this.conversationid = conversationid;
	}

	public String getUserCprIdentifier() {
		return userCprIdentifier;
	}

	public void setUserCprIdentifier(String userCprIdentifier) {
		this.userCprIdentifier = userCprIdentifier;
	}

	public void setUserCprIdentifier(HsuidHeader hsuidHeader) {
		if(hsuidHeader != null) {
			ValidatedHsuidAttributes validatedHsuidAttributes = HealthcareServiceUserIdentificationHeaderUtil.getValidatedHsuidAttributes(hsuidHeader);
			if (validatedHsuidAttributes.isCitizen()) {
				this.userCprIdentifier = validatedHsuidAttributes.getValidatedCitizenHsuidAttributes().getActingUserCivilRegistrationNumber();
			}
			if (validatedHsuidAttributes.isHealthCareProfessional()) {
				this.userCprIdentifier = validatedHsuidAttributes.getValidatedHealthcareProfessionalHsuidAttributes().getActingUserCivilRegistrationNumber();
			} else if (validatedHsuidAttributes.isUnauthorizedProfessionl()) {
				this.userCprIdentifier = validatedHsuidAttributes.getValidatedUnauthorizedProfesionalHsuidAttributes().getActingUserCivilRegistrationNumber();
			}
		}
	}

	public void setItSystemName(String itSystemName) {
		this.itSystemName = itSystemName;
	}

	public SecurityContext.Organisation getOrganisation() {
		return organisation;
	}

	public void setOrganisation(SecurityContext.Organisation organisation) {
		this.organisation = organisation;
	}

	public static JAXBContext getJaxbContext() {
		return jaxbContext;
	}
}
