/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda;

import java.util.Locale;

public class LanguageExt {
    public static String toDanishLanguageName(Locale locale) {
        // The Locale class does not cover all danish language names, and will instead return the english name.
        // Therefore, we manually map the language codes to their danish names
        String l = locale.getLanguage();
        String r = l.equals("aa") ? "Afar" :
                l.equals("ab") ? "Abkhasisk" :
                l.equals("ae") ? "Avestisk" :
                l.equals("af") ? "Afrikaans" :
                l.equals("ak") ? "Akan" :
                l.equals("am") ? "Amharisk" :
                l.equals("an") ? "Aragonisk" :
                l.equals("ar") ? "Arabisk" :
                l.equals("as") ? "Assamesisk" :
                l.equals("av") ? "Avarisk" :
                l.equals("ay") ? "Aymara" :
                l.equals("az") ? "Aserbajdsjansk" :
                l.equals("ba") ? "Basjkirsk" :
                l.equals("be") ? "Hviderussisk" :
                l.equals("bg") ? "Bulgarsk" :
                l.equals("bh") ? "Bihari-sprog" :
                l.equals("bi") ? "Bislama" :
                l.equals("bm") ? "Bambara" :
                l.equals("bn") ? "Bengalsk" :
                l.equals("bo") ? "Tibetansk" :
                l.equals("br") ? "Bretonsk" :
                l.equals("bs") ? "Bosnisk" :
                l.equals("ca") ? "Katalansk" :
                l.equals("ce") ? "Tjetjensk" :
                l.equals("ch") ? "Chamorro" :
                l.equals("co") ? "Korsikansk" :
                l.equals("cr") ? "Cree" :
                l.equals("cs") ? "Tjekkisk" :
                l.equals("cu") ? "Kirkeslavisk" :
                l.equals("cv") ? "Tjuvashisk" :
                l.equals("cy") ? "Walisisk" :
                l.equals("da") ? "Dansk" :
                l.equals("de") ? "Tysk" :
                l.equals("dv") ? "Dhivehi" :
                l.equals("dz") ? "Dzongkha" :
                l.equals("ee") ? "Ewe" :
                l.equals("el") ? "Græsk" :
                l.equals("en") ? "Engelsk" :
                l.equals("eo") ? "Esperanto" :
                l.equals("es") ? "Spansk" :
                l.equals("et") ? "Estisk" :
                l.equals("eu") ? "Baskisk" :
                l.equals("fa") ? "Persisk" :
                l.equals("ff") ? "Fula" :
                l.equals("fi") ? "Finsk" :
                l.equals("fj") ? "Fijiansk" :
                l.equals("fo") ? "Færøsk" :
                l.equals("fr") ? "Fransk" :
                l.equals("fy") ? "Vestfrisisk" :
                l.equals("ga") ? "Irsk" :
                l.equals("gd") ? "Skotsk gælisk" :
                l.equals("gl") ? "Galicisk" :
                l.equals("gn") ? "Guarani" :
                l.equals("gu") ? "Gujarati" :
                l.equals("gv") ? "Manx" :
                l.equals("ha") ? "Hausa" :
                l.equals("he") ? "Hebraisk" :
                l.equals("hi") ? "Hindi" :
                l.equals("ho") ? "Hiri Motu" :
                l.equals("hr") ? "Kroatisk" :
                l.equals("ht") ? "Haitisk" :
                l.equals("hu") ? "Ungarsk" :
                l.equals("hy") ? "Armensk" :
                l.equals("hz") ? "Herero" :
                l.equals("ia") ? "Interlingua" :
                l.equals("id") ? "Indonesisk" :
                l.equals("ie") ? "Interlingue" :
                l.equals("ig") ? "Igbo" :
                l.equals("ii") ? "Nuosu" :
                l.equals("ik") ? "Inupiaq" :
                l.equals("io") ? "Ido" :
                l.equals("is") ? "Islandsk" :
                l.equals("it") ? "Italiensk" :
                l.equals("iu") ? "Inuktitut" :
                l.equals("ja") ? "Japansk" :
                l.equals("jv") ? "Javanesisk" :
                l.equals("ka") ? "Georgisk" :
                l.equals("kg") ? "Kongo" :
                l.equals("ki") ? "Kikuyu" :
                l.equals("kj") ? "Kwanyama" :
                l.equals("kk") ? "Kasakhisk" :
                l.equals("kl") ? "Grønlandsk" :
                l.equals("km") ? "Khmer" :
                l.equals("kn") ? "Kannada" :
                l.equals("ko") ? "Koreansk" :
                l.equals("kr") ? "Kanuri" :
                l.equals("ks") ? "Kashmiri" :
                l.equals("ku") ? "Kurdisk" :
                l.equals("kv") ? "Komi" :
                l.equals("kw") ? "Kornisk" :
                l.equals("ky") ? "Kirgisisk" :
                l.equals("la") ? "Latin" :
                l.equals("lb") ? "Luxembourgsk" :
                l.equals("lg") ? "Ganda" :
                l.equals("li") ? "Limburgisk" :
                l.equals("ln") ? "Lingala" :
                l.equals("lo") ? "Lao" :
                l.equals("lt") ? "Litauisk" :
                l.equals("lu") ? "Luba-Katanga" :
                l.equals("lv") ? "Lettisk" :
                l.equals("mg") ? "Malagasy" :
                l.equals("mh") ? "Marshallesisk" :
                l.equals("mi") ? "Maori" :
                l.equals("mk") ? "Makedonsk" :
                l.equals("ml") ? "Malayalam" :
                l.equals("mn") ? "Mongolsk" :
                l.equals("mr") ? "Marathi" :
                l.equals("ms") ? "Malaysisk" :
                l.equals("mt") ? "Maltesisk" :
                l.equals("my") ? "Burmesisk" :
                l.equals("na") ? "Nauru" :
                l.equals("nb") ? "Norsk bokmål" :
                l.equals("nd") ? "Nord-ndebele" :
                l.equals("ne") ? "Nepalesisk" :
                l.equals("ng") ? "Ndonga" :
                l.equals("nl") ? "Hollandsk" :
                l.equals("nn") ? "Norsk nynorsk" :
                l.equals("no") ? "Norsk" :
                l.equals("nr") ? "Syd-ndebele" :
                l.equals("nv") ? "Navajo" :
                l.equals("ny") ? "Chichewa" :
                l.equals("oc") ? "Occitansk" :
                l.equals("oj") ? "Ojibwe" :
                l.equals("om") ? "Oromo" :
                l.equals("or") ? "Oriya" :
                l.equals("os") ? "Ossetisk" :
                l.equals("pa") ? "Punjabi" :
                l.equals("pi") ? "Pali" :
                l.equals("pl") ? "Polsk" :
                l.equals("ps") ? "Pashto" :
                l.equals("pt") ? "Portugisisk" :
                l.equals("qu") ? "Quechua" :
                l.equals("rm") ? "Retoromansk" :
                l.equals("rn") ? "Rundi" :
                l.equals("ro") ? "Rumænsk" :
                l.equals("ru") ? "Russisk" :
                l.equals("rw") ? "Kinyarwanda" :
                l.equals("sa") ? "Sanskrit" :
                l.equals("sc") ? "Sardisk" :
                l.equals("sd") ? "Sindhi" :
                l.equals("se") ? "Nordsamisk" :
                l.equals("sg") ? "Sango" :
                l.equals("sh") ? "Serbokroatisk" :
                l.equals("si") ? "Singalesisk" :
                l.equals("sk") ? "Slovakisk" :
                l.equals("sl") ? "Slovensk" :
                l.equals("sm") ? "Samoansk" :
                l.equals("sn") ? "Shona" :
                l.equals("so") ? "Somalisk" :
                l.equals("sq") ? "Albansk" :
                l.equals("sr") ? "Serbisk" :
                l.equals("ss") ? "Swati" :
                l.equals("st") ? "Sydsotho" :
                l.equals("su") ? "Sundanesisk" :
                l.equals("sv") ? "Svensk" :
                l.equals("sw") ? "Swahili" :
                l.equals("ta") ? "Tamil" :
                l.equals("te") ? "Telugu" :
                l.equals("tg") ? "Tadsjikisk" :
                l.equals("th") ? "Thai" :
                l.equals("ti") ? "Tigrinya" :
                l.equals("tk") ? "Turkmensk" :
                l.equals("tl") ? "Tagalog" :
                l.equals("tn") ? "Tswana" :
                l.equals("to") ? "Tongansk" :
                l.equals("tr") ? "Tyrkisk" :
                l.equals("ts") ? "Tsonga" :
                l.equals("tt") ? "Tatarisk" :
                l.equals("tw") ? "Twi" :
                l.equals("ty") ? "Tahitisk" :
                l.equals("ug") ? "Uigurisk" :
                l.equals("uk") ? "Ukrainsk" :
                l.equals("ur") ? "Urdu" :
                l.equals("uz") ? "Usbekisk" :
                l.equals("ve") ? "Venda" :
                l.equals("vi") ? "Vietnamesisk" :
                l.equals("vo") ? "Volapük" :
                l.equals("wa") ? "Vallonsk" :
                l.equals("wo") ? "Wolof" :
                l.equals("xh") ? "Xhosa" :
                l.equals("yi") ? "Jiddisch" :
                l.equals("yo") ? "Yoruba" :
                l.equals("za") ? "Zhuang" :
                l.equals("zh") ? "Kinesisk" :
                l.equals("zu") ? "Zulu" :
                null;
        if(r == null) { throw new RuntimeException("Missing mapping for: " + l); }
        return r;
    }
}