/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda;

import org.hl7.v3.EnFamily;
import org.hl7.v3.EnGiven;
import org.hl7.v3.POCDMT000040Organization;
import org.hl7.v3.POCDMT000040Person;

import javax.xml.bind.JAXBElement;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static java.util.Collections.emptyList;
import static java.util.Optional.ofNullable;

public class PdcDataMapperCommon {

    protected static List<String> getPersonGivens(POCDMT000040Person person) {
        return ofNullable(person).map(POCDMT000040Person::getNames).orElse(emptyList())
                .stream()
                .reduce((pn, pn2) -> {
                    throw new IllegalStateException("Multiple name elements for person");
                })
                .map(pn -> pn.getContent()
                        .stream()
                        .filter(o -> o instanceof JAXBElement)
                        .map(o -> ((JAXBElement<?>) o).getValue())
                        .filter(o -> o instanceof EnGiven)
                        .map(o -> ((EnGiven) o).getContent())
                        .flatMap(Collection::stream)
                        .map(Objects::toString)
                        .collect(Collectors.toList()))
                .orElse(emptyList());
    }

    protected static String getPersonFamily(POCDMT000040Person person) {
        return ofNullable(person).map(POCDMT000040Person::getNames).orElse(emptyList())
                .stream()
                .reduce((pn, pn2) -> {
                    throw new IllegalStateException("Multiple name elements for person");
                })
                .flatMap(elements -> elements.getContent()
                        .stream()
                        .filter(o -> o instanceof JAXBElement)
                        .map(o -> ((JAXBElement<?>) o).getValue())
                        .filter(o -> o instanceof EnFamily)
                        .map(o -> ((EnFamily) o).getContent())
                        .flatMap(Collection::stream)
                        .map(Objects::toString)
                        .reduce((pn, pn2) -> {
                            throw new IllegalStateException("Multiple name->family elements for person");
                        }))
                .orElse(null);
    }

    protected static String getOrganizationName(POCDMT000040Organization organization) {
        return organization.getNames()
                .stream()
                .reduce((pn, pn2) -> {
                    throw new IllegalStateException("Multiple name elements for organization");
                })
                .flatMap(on -> on.getContent()
                        .stream()
                        .reduce((serializable, serializable2) -> {
                            throw new IllegalStateException("Multiple elements for string content");
                        })
                        .map(Objects::toString))
                .orElse(null);
    }

    protected static String buildAddressLine1(String streetName, String streetBuildingIdentifier, String floor, String suiteId) {
        return streetName + withSpace(streetBuildingIdentifier) + withSpace(floor) + withSpace(suiteId);
    }

    protected static String withSpace(String value) { return ofNullable(value).map(v -> " " + v).orElse(""); }
}
