/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.assembler;

import dk.stamkort.util.FaultCode;
import dk.stamkort.util.FaultUtil;

import java.util.List;

public class SourceDataValidator {

    public static void failIfNull(Object o) {
        if (o == null) {
            FaultUtil.throwFSKFault(FaultCode.MAPPING_ERROR_MISSING_DATA);
        }
    }

    public static void failIfEmpty(List<?> list) {
        if (list == null || list.isEmpty()) {
            FaultUtil.throwFSKFault(FaultCode.MAPPING_ERROR_MISSING_DATA);
        }
    }

    public static void failIfWrongSize(List<?> list, int minSize, int maxSize) {
        if (list == null) {
            FaultUtil.throwFSKFault(FaultCode.MAPPING_ERROR_MISSING_DATA);
        } else if (list.size() < minSize || list.size() > maxSize) {
            FaultUtil.throwFSKFault(FaultCode.MAPPING_ERROR_INVALID_DATA);
        }
    }
}
