/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.assembler.mapper;

import dk.stamkort.webservice.hl7cda.model.*;
import dk.stamkort.webservice.hl7cda.model.v3.DentistTypedInV3;
import dk.stamkort.webservice.hl7cda.model.v3.LanguageTypedInV3;
import dk.stamkort.webservice.hl7cda.model.v3.PatientContactTypedInV3;
import dk.stamkort.webservice.hl7cda.model.v3.RelativeTypedInV3;
import dk.stamkort.webservice.hl7cda.model.v3.TempAddrTypedInV3;
import org.hl7.v3.*;

import javax.xml.bind.JAXBElement;
import java.io.Serializable;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class PDC_3_0NarrativeTextMapper {

    private static final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss Z"); // Output sample: 2017-10-19 14:42:00 +0200
    private static final DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    public static final ObjectFactory o = new ObjectFactory();

    public static List<Serializable> createTextContentElements(PrimaryPersonInformationHolder primaryPersonInformation,
                                                               List<CustodyHolder> custodyHolderList,
                                                               CoverageGroupHolder coverageGroupHolder,
                                                               OrganDonorRegistrationHolder organDonorRegistrationHolder,
                                                               TreatmentWillRegistrationHolder treatmentWillRegistrationHolder,
                                                               LivingWillRegistrationHolder livingWillRegistrationHolder,
                                                               Optional<LanguageTypedInV3> languageTypedInHolderList,
                                                               Optional<TempAddrTypedInV3> tempAddrTypedInHolderList,
                                                               Optional<DentistTypedInV3> dentistTypedInHolder,
                                                               Optional<PatientContactTypedInV3> patientContactTypedInHolderList,
                                                               List<RelativeTypedInV3> relativeTypedInHolderList,
                                                               Optional<Boolean> hasNoResuscitation
    ) {
        List<Serializable> textContentElements = new ArrayList<>();

        // paragraph
        textContentElements.add(mapParagraph());

        // table
        textContentElements.add(
                mapTable(
                        primaryPersonInformation,
                        custodyHolderList,
                        coverageGroupHolder,
                        organDonorRegistrationHolder,
                        treatmentWillRegistrationHolder,
                        livingWillRegistrationHolder,
                        languageTypedInHolderList,
                        tempAddrTypedInHolderList,
                        dentistTypedInHolder,
                        patientContactTypedInHolderList,
                        relativeTypedInHolderList,
                        hasNoResuscitation
                ));

        return textContentElements;
    }

    private static JAXBElement<StrucDocParagraph> mapParagraph() {
        StrucDocParagraph paragraph = o.createStrucDocParagraph();
        paragraph.getContent().add("Øvrige oplysninger");

        return o.createStrucDocTextParagraph(paragraph);
    }

    private static JAXBElement<StrucDocTable> mapTable(PrimaryPersonInformationHolder primaryPersonInformation,
                                                       List<CustodyHolder> custodyHolderList,
                                                       CoverageGroupHolder coverageGroupHolder,
                                                       OrganDonorRegistrationHolder organDonorRegistrationHolder,
                                                       TreatmentWillRegistrationHolder treatmentWillRegistrationHolder,
                                                       LivingWillRegistrationHolder livingWillRegistrationHolder,
                                                       Optional<LanguageTypedInV3> languageTypedInHolderList,
                                                       Optional<TempAddrTypedInV3> tempAddrTypedInHolderList,
                                                       Optional<DentistTypedInV3> dentistTypedInHolder,
                                                       Optional<PatientContactTypedInV3> patientContactTypedInHolderList,
                                                       List<RelativeTypedInV3> relativeTypedInHolderList,
                                                       Optional<Boolean> hasNoResuscitation
    ) {
        StrucDocTable table = o.createStrucDocTable();
        table.setWidth("100%");
        table.getTbodies().add(
                mapTbody(primaryPersonInformation,
                        custodyHolderList,
                        coverageGroupHolder,
                        organDonorRegistrationHolder,
                        treatmentWillRegistrationHolder,
                        livingWillRegistrationHolder,
                        languageTypedInHolderList,
                        tempAddrTypedInHolderList,
                        dentistTypedInHolder,
                        patientContactTypedInHolderList,
                        relativeTypedInHolderList,
                        hasNoResuscitation
                ));

        return o.createStrucDocTextTable(table);
    }

    private static StrucDocTbody mapTbody(PrimaryPersonInformationHolder primaryPersonInformation,
                                          List<CustodyHolder> custodyHolderList,
                                          CoverageGroupHolder coverageGroupHolder,
                                          OrganDonorRegistrationHolder organDonorRegistrationHolder,
                                          TreatmentWillRegistrationHolder treatmentWillRegistrationHolder,
                                          LivingWillRegistrationHolder livingWillRegistrationHolder,
                                          Optional<LanguageTypedInV3> languageTypedInHolderList,
                                          Optional<TempAddrTypedInV3> tempAddrTypedInHolderList,
                                          Optional<DentistTypedInV3> dentistTypedInHolder,
                                          Optional<PatientContactTypedInV3> patientContactTypedInHolderList,
                                          List<RelativeTypedInV3> relativeTypedInHolderList,
                                          Optional<Boolean> hasNoResuscitation
    ) {
        StrucDocTbody tbody = o.createStrucDocTbody();
        tbody.getTrs().add(mapTr_header());

        // ChildCustody/CustodyBy
        custodyHolderList.forEach(custodyHolder ->
                tbody.getTrs().add(mapTr_Custody(custodyHolder)));

        // CitizenNameAddr
        tbody.getTrs().add(mapTr_CitizenNameAddr(primaryPersonInformation));

        // CoverageGroup
        tbody.getTrs().add(mapTr_CoverageGroup(coverageGroupHolder));

        // OrganDonorRegistration
        tbody.getTrs().add(mapTr_OrganDonorRegistration(organDonorRegistrationHolder));

        // TreatmentWillRegistration
        tbody.getTrs().add(mapTr_TreatmentWillRegistration(treatmentWillRegistrationHolder));

        // LivingWillRegistration
        tbody.getTrs().add(mapTr_LivingWillRegistration(livingWillRegistrationHolder));

        // LanguageTypedIn
        languageTypedInHolderList.ifPresent(languageTypedInHolder ->
                tbody.getTrs().add(mapTr_LanguageTypedIn(languageTypedInHolder)));

        // TempAddrTypedIn
        tempAddrTypedInHolderList.ifPresent(tempAddrTypedInHolder ->
                tbody.getTrs().add(mapTr_TempAddrTypedIn(tempAddrTypedInHolder)));

        // DentistTypedIn
        dentistTypedInHolder.ifPresent(holder -> tbody.getTrs().add(mapTr_DentistTypedIn(holder)));

        // PatientContactTypedIn
        patientContactTypedInHolderList.ifPresent(patientContactTypedInHolder ->
                tbody.getTrs().add(mapTr_PatientContactTypedIn(patientContactTypedInHolder)));

        // RelativeTypedIn
        relativeTypedInHolderList.forEach(relativeTypedInHolder ->
                tbody.getTrs().add(mapTr_RelativeTypedIn(relativeTypedInHolder)));

        // HasNoResuscitation
        hasNoResuscitation.ifPresent(hasNoResuscitationValue ->
                tbody.getTrs().add(mapTr_HasNoResuscitation(hasNoResuscitationValue)));

        return tbody;
    }

    private static StrucDocTr mapTr_Custody(CustodyHolder custodyHolder) {
        return mapTr(
                map4TdWithList(
                        custodyHolder.getCode().getDisplayName(),
                        Arrays.asList(
                                "Relation: " + custodyHolder.getRelation().getDisplayName(),
                                "CPR: " + custodyHolder.getCpr(),
                                "Navn: " + String.join(" ", custodyHolder.getGivens()) + " " + custodyHolder.getFamily()),
                        "CPR",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_CitizenNameAddr(PrimaryPersonInformationHolder primaryPersonInformation) {
        List<String> listItems = new ArrayList<>();
        listItems.add(String.join(" ", primaryPersonInformation.getGivens()) + " " + primaryPersonInformation.getFamily());
        listItems.addAll(primaryPersonInformation.getAddress().getAddressLines());
        listItems.add(primaryPersonInformation.getAddress().getPostalCode() + " " + primaryPersonInformation.getAddress().getCity());
        listItems.add(primaryPersonInformation.getAddress().getCountry());

        return mapTr(
                map4TdWithList(
                        "Borgerens navn og adresse",
                        listItems,
                        "CPR",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_CoverageGroup(CoverageGroupHolder coverageGroupHolder) {
        return mapTr(
                map4TdWithList(
                        "Sygesikringsgruppe",
                        Collections.singletonList(coverageGroupHolder.getGroup()),
                        "CPR",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_OrganDonorRegistration(OrganDonorRegistrationHolder organDonorRegistrationHolder) {
        return mapTr(
                map4TdWithList(
                        "Registreret organdonor",
                        Collections.singletonList(organDonorRegistrationHolder.isHasRegistration() ? "Ja" : "Nej"),
                        "Dansk Center For Organdonation",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_TreatmentWillRegistration(TreatmentWillRegistrationHolder treatmentWillRegistrationHolder) {
        return mapTr(
                map4TdWithList(
                        "Registreret behandlingstestamente",
                        Collections.singletonList(treatmentWillRegistrationHolder.isHasRegistration() ? "Ja" : "Nej"),
                        "Sundhedsdatastyrelsen",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_LivingWillRegistration(LivingWillRegistrationHolder livingWillRegistrationHolder) {
        return mapTr(
                map4TdWithList(
                        "Registreret livstestamente",
                        Collections.singletonList(livingWillRegistrationHolder.isHasRegistration() ? "Ja" : "Nej"),
                        "Sundhedsdatastyrelsen",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_LanguageTypedIn(LanguageTypedInHolder languageTypedInHolder) {
        return mapTr(
                map4TdWithList(
                        "Talt sprog, indtastet",
                        Collections.singletonList(languageTypedInHolder.getLanguageName()),
                        String.join(" ", languageTypedInHolder.getAuthorGivens()) + " " + languageTypedInHolder.getAuthorFamily(),
                        dateTimeFormatter.format(languageTypedInHolder.getAuthorTime())));
    }

    private static StrucDocTr mapTr_TempAddrTypedIn(TempAddrTypedInHolder tempAddrTypedInHolder) {
        List<String> listItems = new ArrayList<>();

        // PeriodFrom is mandatory. PeriodTo is optional.
        if(tempAddrTypedInHolder.getPeriodTo()  != null) {
            listItems.add("Periode: " + dateFormatter.format(tempAddrTypedInHolder.getPeriodFrom()) + " - " + dateFormatter.format(tempAddrTypedInHolder.getPeriodTo()));
        }
        else {
            listItems.add("Periode fra: " + dateFormatter.format(tempAddrTypedInHolder.getPeriodFrom()));
        }
        listItems.addAll(tempAddrTypedInHolder.getAddress().getAddressLines());
        listItems.add(tempAddrTypedInHolder.getAddress().getPostalCode() + " " + tempAddrTypedInHolder.getAddress().getCity());
        listItems.add(tempAddrTypedInHolder.getAddress().getCountry());

        return mapTr(
                map4TdWithList(
                        "Midlertidig adresse, indtastet",
                        listItems,
                        String.join(" ", tempAddrTypedInHolder.getAuthorGivens()) + " " + tempAddrTypedInHolder.getAuthorFamily(),
                        dateTimeFormatter.format(tempAddrTypedInHolder.getAuthorTime())));
    }

    private static StrucDocTr mapTr_DentistTypedIn(DentistTypedInV3 dentistTypedInHolder) {
        List<String> listItems = new ArrayList<>();
        dentistTypedInHolder.getName().ifPresent(listItems::add);
        listItems.add(dentistTypedInHolder.getOrganizationName());
        listItems.addAll(dentistTypedInHolder.getAddress().getAddressLines());
        if (dentistTypedInHolder.getAddress().getPostalCode() != null || dentistTypedInHolder.getAddress().getCity() != null) {
            listItems.add(dentistTypedInHolder.getAddress().getPostalCode() + " " + dentistTypedInHolder.getAddress().getCity());
        }
        if(dentistTypedInHolder.getAddress().getCountry() != null) {
            listItems.add(dentistTypedInHolder.getAddress().getCountry());
        }
        dentistTypedInHolder.getPhone()
                .ifPresent(p -> listItems.add(FormatHelper.formatPhoneNumber(p.getValue())));

        return mapTr(
                map4TdWithList(
                        "Tandlæge indtastet",
                        listItems,
                        String.join(" ", dentistTypedInHolder.getAuthorGivens()) + " " + dentistTypedInHolder.getAuthorFamily(),
                        dateTimeFormatter.format(dentistTypedInHolder.getAuthorTime())));
    }

    private static StrucDocTr mapTr_HasNoResuscitation(boolean hasNoResuscitation) {
        return mapTr(
                map4TdWithList(
                        "Fravalg af genoplivning ved hjertestop",
                        Collections.singletonList(hasNoResuscitation ? "Aktiv" : "Inaktiv"),
                        "Sundhedsdatastyrelsen",
                        "Ikke oplyst"));
    }

    private static StrucDocTr mapTr_PatientContactTypedIn(PatientContactTypedInHolder patientContactTypedInHolder) {
        List<String> listItems = new ArrayList<>();
        patientContactTypedInHolder.getPhones()
                .forEach(phoneHolder -> listItems.add(FormatHelper.formatPhoneNumber(phoneHolder.getValue())));
        patientContactTypedInHolder.getEmailAddresses()
                .forEach(emailAddressHolder -> listItems.add(emailAddressHolder.getValue()));

        return mapTr(
                map4TdWithList(
                        "Kontaktoplysninger, indtastet",
                        listItems,
                        String.join(" ", patientContactTypedInHolder.getAuthorGivens()) + " " + patientContactTypedInHolder.getAuthorFamily(),
                        dateTimeFormatter.format(patientContactTypedInHolder.getAuthorTime())));
    }

    private static StrucDocTr mapTr_RelativeTypedIn(RelativeTypedInHolder relativeTypedInHolder) {
        List<String> listItems = new ArrayList<>();
        listItems.add(String.join(" ", relativeTypedInHolder.getGivens()) + " " + relativeTypedInHolder.getFamily());
        listItems.add(relativeTypedInHolder.getRelation().getDisplayName());
        relativeTypedInHolder.getPhoneHolders()
                .forEach(phoneHolder -> listItems.add(FormatHelper.formatPhoneNumber(phoneHolder.getValue())));
        listItems.add(relativeTypedInHolder.getNote());

        return mapTr(
                map4TdWithList(
                        "Pårørende, indtastet",
                        listItems,
                        String.join(" ", relativeTypedInHolder.getAuthorGivens()) + " " + relativeTypedInHolder.getAuthorFamily(),
                        dateTimeFormatter.format(relativeTypedInHolder.getAuthorTime())));
    }

    private static StrucDocTr mapTr_header() {
        StrucDocTr tr = o.createStrucDocTr();

        StrucDocTh th1 = o.createStrucDocTh();
        th1.getContent().add("Oplysning");

        StrucDocTh th2 = o.createStrucDocTh();
        th2.getContent().add("Værdi");

        StrucDocTh th3 = o.createStrucDocTh();
        th3.getContent().add("Kilde/Indtastet af");

        StrucDocTh th4 = o.createStrucDocTh();
        th4.getContent().add("Sidst ændret/Indtastet den");

        tr.getThsAndTds().add(th1);
        tr.getThsAndTds().add(th2);
        tr.getThsAndTds().add(th3);
        tr.getThsAndTds().add(th4);

        return tr;
    }

    private static StrucDocTr mapTr(List<? extends Serializable> tdList) {
        StrucDocTr tr = o.createStrucDocTr();

        tr.getThsAndTds().addAll(tdList);

        return tr;
    }

    private static List<StrucDocTd> map4TdWithList(String cell1, List<String> listItems, String cell3, String cell4) {
        List<StrucDocTd> tdList = new ArrayList<>();

        StrucDocTd td1 = o.createStrucDocTd();
        td1.getContent().add(cell1);

        StrucDocTd td2 = o.createStrucDocTd();
        td2.getContent().add(mapList(listItems));

        StrucDocTd td3 = o.createStrucDocTd();
        td3.getContent().add(cell3);

        StrucDocTd td4 = o.createStrucDocTd();
        td4.getContent().add(cell4);

        tdList.add(td1);
        tdList.add(td2);
        tdList.add(td3);
        tdList.add(td4);

        return tdList;
    }

    private static JAXBElement<StrucDocList> mapList(List<String> listItems) {
        StrucDocList list = o.createStrucDocList();

        listItems.forEach(itemContent -> {
            StrucDocItem item = o.createStrucDocItem();
            item.getContent().add(itemContent);
            list.getItems().add(item);
        });

        return o.createStrucDocTdList(list);
    }
}
