/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.model;

import java.time.LocalDate;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;

public class PrimaryPersonInformationHolder {

    private String cpr;
    private List<String> givens = new ArrayList<>();
    private String family;
    private String gender;
    private ZonedDateTime birthDate;
    private AddressHolder address;

    public String getCpr() {
        return cpr;
    }

    public void setCpr(String cpr) {
        this.cpr = cpr;
    }

    public List<String> getGivens() {
        return givens;
    }

    public void addGiven(String given) {
        this.givens.add(given);
    }

    public String getFamily() {
        return family;
    }

    public void setFamily(String family) {
        this.family = family;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public ZonedDateTime getBirthDate() {
        return birthDate;
    }

    // Denne er kun brugt for at understøtter v2 som ikke kan håndtere ZonedDateTime
    // da det ikke vil gå godt for anvender som er vandt til noget andet
    public LocalDate getBirthDateAsLocalDate() {
        return birthDate.toLocalDate();
    }

    //Medcom beslutning om at udekendte tidspunkter altid skal returneres som 000000+0000
    public void setBirthDate(LocalDate birthTime) {
        this.birthDate = birthTime.atStartOfDay(ZoneOffset.UTC);
    }

    //Medcom beslutning om at udekendte tidspunkter altid skal returneres som 000000+0000
    public void setBirthDate(ZonedDateTime birthTime) {
        this.birthDate = birthTime.toLocalDate().atStartOfDay(ZoneOffset.UTC);
    }

    public AddressHolder getAddress() {
        return address;
    }

    public void setAddress(AddressHolder address) {
        this.address = address;
    }
}
