/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.model.v3;

import dk.stamkort.webservice.hl7cda.model.AddressHolder;
import dk.stamkort.webservice.hl7cda.model.PhoneHolder;

import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static java.util.Optional.*;

public class DentistTypedInV3 {
    public DentistTypedInV3(AssignedAuthor a) { this.a = a; }

    public AssignedAuthor getAssignedAuthor() { return a; }

    public void setPhone(PhoneHolder p) {
        if(p.getValue().matches("tel:\\+?[-0-9().]+")) { phone = of(p); }
    }

    private Optional<String> name = empty();
    private AddressHolder address;
    private String organizationName;
    private Optional<PhoneHolder> phone = empty();
    private ZonedDateTime authorTime;
    private List<String> authorGivens = new ArrayList<>();
    private String authorFamily;
    private String yder;
    private final AssignedAuthor a;
    private String idExtension;

    public Optional<String> getName() {
        return name;
    }

    public void setName(String name) {
        this.name = ofNullable(name);
    }

    public AddressHolder getAddress() {
        return address;
    }

    public void setAddress(AddressHolder address) {
        this.address = address;
    }

    public Optional<PhoneHolder> getPhone() {
        return phone;
    }

    public void setPhone(Optional<PhoneHolder> phone) {
        this.phone = phone;
    }

    public ZonedDateTime getAuthorTime() {
        return authorTime;
    }

    public void setAuthorTime(ZonedDateTime authorTime) {
        this.authorTime = authorTime;
    }

    public List<String> getAuthorGivens() {
        return authorGivens;
    }

    public void addAuthorGiven(String authorGiven) {
        this.authorGivens.add(authorGiven);
    }

    public String getAuthorFamily() {
        return authorFamily;
    }

    public void setAuthorFamily(String authorFamily) {
        this.authorFamily = authorFamily;
    }

    public String getYder() {
        return yder;
    }

    public void setYder(String yder) {
        this.yder = yder;
    }

    public String getOrganizationName() {
        return organizationName;
    }

    public void setOrganizationName(String organizationName) {
        this.organizationName = organizationName;
    }

    public void setIdExtension(String extension) {
        this.idExtension = extension;
    }

    public String getIdExtension() {
        return idExtension;
    }
}