/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.model.v3;

import dk.stamkort.webservice.hl7cda.model.*;

import java.util.List;
import java.util.Optional;

public class PdcV3Data {

    private PrimaryPersonInformationHolder primaryPersonInformation;
    private Optional<ProviderOrganizationHolder> providerOrganizationHolder;
    private List<CustodyHolder> custodyHolderList;
    private CoverageGroupHolder coverageGroupHolder;
    private OrganDonorRegistrationHolder organDonorRegistrationHolder;
    private TreatmentWillRegistrationHolder treatmentWillRegistrationHolder;
    private LivingWillRegistrationHolder livingWillRegistrationHolder;
    private Optional<LanguageTypedInV3> languageTypedInV3;
    private Optional<TempAddrTypedInV3> tempAddrTypedInV3;
    private Optional<DentistTypedInV3> dentistTypedInV3;
    private Optional<PatientContactTypedInV3> patientContactTypedInV3;
    private List<RelativeTypedInV3> relativeTypedInV3;
    public final Optional<Boolean> hasNoResuscitation;

    public PdcV3Data(Optional<Boolean> hasNoResuscitation) {
        this.hasNoResuscitation = toOptional(hasNoResuscitation, false);
    }

    private <T> Optional<T> toOptional(Optional<T> in, T t){
        return in != null && in.isPresent() ? in : Optional.of(t);
    }

    public PrimaryPersonInformationHolder getPrimaryPersonInformation() { return primaryPersonInformation; }

    public PdcV3Data setPrimaryPersonInformation(PrimaryPersonInformationHolder primaryPersonInformation) {
        this.primaryPersonInformation = primaryPersonInformation;
        return this;
    }

    public Optional<ProviderOrganizationHolder> getProviderOrganizationHolder() { return providerOrganizationHolder; }

    public PdcV3Data setProviderOrganizationHolder(Optional<ProviderOrganizationHolder> providerOrganizationHolder) {
        this.providerOrganizationHolder = providerOrganizationHolder;
        return this;
    }

    public List<CustodyHolder> getCustodyHolderList() { return custodyHolderList; }

    public PdcV3Data setCustodyHolderList(List<CustodyHolder> custodyHolderList) {
        this.custodyHolderList = custodyHolderList;
        return this;
    }

    public CoverageGroupHolder getCoverageGroupHolder() { return coverageGroupHolder; }

    public PdcV3Data setCoverageGroupHolder(CoverageGroupHolder coverageGroupHolder) {
        this.coverageGroupHolder = coverageGroupHolder;
        return this;
    }

    public OrganDonorRegistrationHolder getOrganDonorRegistrationHolder() { return organDonorRegistrationHolder; }

    public PdcV3Data setOrganDonorRegistrationHolder(OrganDonorRegistrationHolder organDonorRegistrationHolder) {
        this.organDonorRegistrationHolder = organDonorRegistrationHolder;
        return this;
    }

    public TreatmentWillRegistrationHolder getTreatmentWillRegistrationHolder() {
        return treatmentWillRegistrationHolder;
    }

    public PdcV3Data setTreatmentWillRegistrationHolder(TreatmentWillRegistrationHolder treatmentWillRegistrationHolder) {
        this.treatmentWillRegistrationHolder = treatmentWillRegistrationHolder;
        return this;
    }

    public LivingWillRegistrationHolder getLivingWillRegistrationHolder() { return livingWillRegistrationHolder; }

    public PdcV3Data setLivingWillRegistrationHolder(LivingWillRegistrationHolder livingWillRegistrationHolder) {
        this.livingWillRegistrationHolder = livingWillRegistrationHolder;
        return this;
    }

    public Optional<LanguageTypedInV3> getLanguageTypedInV3() { return languageTypedInV3; }

    public PdcV3Data setLanguageTypedInV3(Optional<LanguageTypedInV3> languageTypedInV3) {
        this.languageTypedInV3 = languageTypedInV3;
        return this;
    }

    public Optional<TempAddrTypedInV3> getTempAddrTypedInV3() { return tempAddrTypedInV3; }

    public PdcV3Data setTempAddrTypedInV3(Optional<TempAddrTypedInV3> tempAddrTypedInV3) {
        this.tempAddrTypedInV3 = tempAddrTypedInV3;
        return this;
    }

    public Optional<DentistTypedInV3> getDentistTypedInV3() { return dentistTypedInV3; }

    public PdcV3Data setDentistTypedInV3(Optional<DentistTypedInV3> dentistTypedInV3) {
        this.dentistTypedInV3 = dentistTypedInV3;
        return this;
    }

    public Optional<PatientContactTypedInV3> getPatientContactTypedInV3() { return patientContactTypedInV3; }

    public PdcV3Data setPatientContactTypedInV3(Optional<PatientContactTypedInV3> patientContactTypedInV3) {
        this.patientContactTypedInV3 = patientContactTypedInV3;
        return this;
    }

    public List<RelativeTypedInV3> getRelativeTypedInV3() { return relativeTypedInV3; }

    public PdcV3Data setRelativeTypedInV3(List<RelativeTypedInV3> relativeTypedInV3) {
        this.relativeTypedInV3 = relativeTypedInV3;
        return this;
    }
}
