/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.ondemand;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.context.support.GenericApplicationContext;

import dk.stamkort.FskContextListener;

public abstract class AbstractAutowiredWS {
	
    private static final Logger LOGGER = LogManager.getLogger(AbstractAutowiredWS.class);

    @Resource
    protected WebServiceContext context;

    @PostConstruct
    public void init() throws ServletException {
    	LOGGER.info("Initializing WS");
        ServletContext servletContext = (ServletContext) context.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
        Object o = servletContext.getAttribute(FskContextListener.ANNOTATION_CONFIG_CONTEXT);
        if(o instanceof GenericApplicationContext) {
            ((GenericApplicationContext)o).getAutowireCapableBeanFactory().autowireBean(this);
        }
    }

    public void setWebServiceContext(WebServiceContext webServiceContext) {
    	this.context = webServiceContext;
    }

    public SLALogItem getSlaLogger() {
        HttpServletRequest servletRequest = (HttpServletRequest) context.getMessageContext().get(MessageContext.SERVLET_REQUEST);
        return (SLALogItem) servletRequest.getAttribute(SLALoggingServletFilter.CURRENT_LOGITEM_REQUEST_KEY);
    }
}
