/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.health;

import org.junit.Rule;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;

import java.time.Instant;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class LastStateHealthIndicatorTest {

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private LastStateHealthReporter lastStateHealthReporter;

    @InjectMocks
    LastStateHealthIndicator.Builder lastStateHealthIndicatorBuilder;

    private LastStateHealthIndicator lastStateHealthIndicator;

    @Test
    public void testHealth_UP() {
        // Arrange
        Instant timeOfLastExecution = Instant.now();

        when(lastStateHealthReporter.lastExecutionSucceeded()).thenReturn(true);
        when(lastStateHealthReporter.timeOfLastExecution()).thenReturn(timeOfLastExecution);
        lastStateHealthIndicator = lastStateHealthIndicatorBuilder.build("Test");

        // Act
        int health = lastStateHealthIndicator.getStatusCode();

        // Assert
        assertEquals(200, health);
        assertTrue(lastStateHealthIndicator.getStatusText().contains("timeOfLastExecution"));
    }

    @Test
    public void testHealth_NotRunYet() {
        // Arrange
        lastStateHealthIndicator = lastStateHealthIndicatorBuilder.build("Test");
        when(lastStateHealthReporter.lastExecutionSucceeded()).thenReturn(null);

        // Act
        int health = lastStateHealthIndicator.getStatusCode();

        // Assert
        assertEquals(200, health);
        assertFalse(lastStateHealthIndicator.getStatusText().contains("timeOfLastExecution"));
    }

    @Test
    public void testHealth_NEEDS_ATTENTION() {
        // Arrange
        Instant timeOfLastExecution = Instant.now();
        Exception lastException = new Exception("something");

        when(lastStateHealthReporter.lastExecutionSucceeded()).thenReturn(false);
        when(lastStateHealthReporter.timeOfLastExecution()).thenReturn(timeOfLastExecution);
        lastStateHealthIndicator = lastStateHealthIndicatorBuilder.build("Test");

        // Act
        int health = lastStateHealthIndicator.getStatusCode();

        // Assert
        assertEquals(203, health);
        assertTrue(lastStateHealthIndicator.getStatusText().contains("timeOfLastExecution"));
    }
}
