/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import static org.junit.Assert.assertNull;

import javax.xml.ws.Endpoint;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import dk.stamkort.conf.FskConfig;
import dk.stamkort.conf.TestDAOConfig;
import dk.stamkort.domain.YderVO;
import dk.stamkort.util.RequestContextEnricher;
import dk.stamkort.util.RequestContextVO;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = { TestDAOConfig.class, FskConfig.class })
public class YderClientTest {

    @Mock
    private RequestContextVO requestContextVO;

    @Autowired
    private YderClient yderClient;

    @Autowired
    private RequestContextEnricher requestContextEnricher;

    @Before
    public void setup() {
    	Mockito.when(requestContextVO.getRequestContextEnricher()).thenReturn(requestContextEnricher);
    }
    
    @Test
    public void call() throws DGWSClientException {
        Resource resource = new ClassPathResource("testfiles/HasYder_response_empty_yder_with_envelope.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish("http://localhost:8085/", testServer);

        YderVO response = yderClient.call("1111111", requestContextVO);
        endpoint.stop();
        assertNull(response);
    }
}