/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations.minlog;

import java.time.Instant;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import dk.nsi.common.util.SLALoggingDataHandler;
import dk.nsp.minlog.api.RegisterService;
import dk.nsp.minlog.domain.RegistrationResult;
import dk.stamkort.domain.OrganizationTypeVO;
import dk.stamkort.util.RequestContextVO;

@RunWith(MockitoJUnitRunner.class)
public class MinLogClientTest {

	@Mock
	RegisterService registerService;

	@Mock
	RequestContextVO context;


	public MinLogClient subject;


	@Before
	public void setup() throws MinLogException {
		RegistrationResult registrationResult = new RegistrationResult();
		
		Mockito.when(context.getUserType()).thenReturn(RequestContextVO.UserType.SystemUser);
		Mockito.when(registerService.addRegistrations(Mockito.anyList())).thenReturn(registrationResult);
	}

	@Test
	public void testMinLogEnabledForSystemUserLaverKaldTilMinlogForSystemBruger() throws MinLogException {

		// Given
		subject = new MinLogClient(registerService, true);
		//Mockito.when(context.getSecurityData()).thenReturn(securityData);
		MinLogData minLogData = new MinLogData("unitTestActivity", Instant.now(), "12345678901234567890", "Regionshospital Randers, Akutcenter", OrganizationTypeVO.SKS, "0309691444", null, "testcpr", null, "FSK integration test");

		// When
		subject.logDataAdd(context, minLogData);
		
		// Then
		Mockito.verify(registerService).addRegistrations(Mockito.anyList());
	}

	@Test
	public void testMinLogDisabledForSystemUserSkipperKaldTilMinlogForSystemBruger() throws MinLogException {

		// Given
		subject = new MinLogClient(registerService, false);
		//Mockito.when(context.getSecurityData()).thenReturn(securityData);
		MinLogData minLogData = new MinLogData("unitTestActivity", Instant.now(), "12345678901234567890", "Regionshospital Randers, Akutcenter", OrganizationTypeVO.SKS, "0309691444", null, "testcpr", null, "FSK integration test");

		// When
		subject.logDataAdd(context, minLogData);

		// Then
		Mockito.verifyZeroInteractions(registerService);
	}

	@Test
	public void testMinLogDisabledForSystemUserSkipperIkkeKaldTilMinlogForAlmBruger() throws MinLogException {

		// Given
		subject = new MinLogClient(registerService, false);
		Mockito.when(context.getUserType()).thenReturn(RequestContextVO.UserType.HealthCareProfessional);
		SLALoggingDataHandler slaLoggingDataHandler = Mockito.mock(SLALoggingDataHandler.class);
		//Mockito.when(context.getSecurityData()).thenReturn(securityData);
		MinLogData minLogData = new MinLogData("unitTestActivity", Instant.now(), "12345678901234567890", "Regionshospital Randers, Akutcenter", OrganizationTypeVO.SKS, "0309691444", null, "testcpr", null, "FSK integration test");

		// When
		subject.logDataAdd(context, minLogData);

		// Then
		Mockito.verify(registerService).addRegistrations(Mockito.anyList());
	}


}