/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

import javax.sql.DataSource;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import dk.stamkort.conf.TestDAOConfig;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = { TestDAOConfig.class })
public class DataSourceHealthIndicatorTest {

	
	@Autowired
	@Qualifier("fsk")
	public DataSource ds;
	
	private DataSourceHealthIndicator subject;
	
	@Before
	public void setup() {
		this.subject = new DataSourceHealthIndicator("test", ds);
	}
	
	@Test
	public void testConnectionSuccess() {
		// Given
		
		// When
		boolean ishealthy = subject.isHealthy();
		
		// then
		Assert.assertTrue(ishealthy);
		Assert.assertEquals(200, subject.getStatusCode());
		Assert.assertEquals("Forbindelse til database test ok", subject.getStatusText());
	}
	
	@Test
	public void testConnectionFailure() {
		// Given
		DataSource faulty = Mockito.mock(DataSource.class);
		this.subject = new DataSourceHealthIndicator("test", faulty);
		
		// When
		boolean ishealthy = subject.isHealthy();
		
		// then
		Assert.assertFalse(ishealthy);
		Assert.assertEquals(500, subject.getStatusCode());
		Assert.assertTrue(subject.getStatusText().startsWith("Forbindelse til database test fejler: Failed to obtain JDBC Connection: DataSource returned null from getConnection()"));
	}

}
