/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class XmlEnhancerTest {
    @Test
    public void testReplace() {
        String input = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\" xmlns=\"urn:hl7-org:v3\" xmlns:ns2=\"urn:hl7-org:sdtc\" xmlns:ns3=\"urn:hl7-org:fsk\"></ClinicalDocument>";

        String result = new XmlEnhancer().enhance(input);

        assertEquals("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\" xmlns=\"urn:hl7-org:v3\" xmlns:ns2=\"urn:hl7-org:sdtc\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:hl7-org:v3 http://svn.medcom.dk/svn/releases/Standarder/HL7/Generic/Schema/CDA_SDTC.xsd\"></ClinicalDocument>", result);
    }

    @Test
    public void testReplaceSomeThingNotNs() {
        String input = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\" xmlns=\"urn:hl7-org:v3\" xmlns:ns2=\"urn:hl7-org:sdtc\" xmlns:SomeThingElse=\"urn:hl7-org:fsk\"></ClinicalDocument>";

        String result = new XmlEnhancer().enhance(input);

        assertEquals("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\" xmlns=\"urn:hl7-org:v3\" xmlns:ns2=\"urn:hl7-org:sdtc\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:hl7-org:v3 http://svn.medcom.dk/svn/releases/Standarder/HL7/Generic/Schema/CDA_SDTC.xsd\"></ClinicalDocument>", result);
    }

    @Test
    public void testReplaceNotFound() {
        String input = "some text";

        String result = new XmlEnhancer().enhance(input);

        assertEquals(input, result);
    }
}
