/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.assembler;

import dk.stamkort.conf.FskConfig;
import dk.stamkort.conf.TestDAOConfig;
import dk.stamkort.webservice.hl7cda.assembler.mapper.FormatHelper;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import static org.springframework.test.util.AssertionErrors.assertEquals;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = { TestDAOConfig.class, FskConfig.class })
public class FormatHelperTest {

    @Test
    public void testCorrectFormat(){
        String phoneNumber = "tel:+45501634";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correct number is not formatted.",phoneNumber, formattedNumber);
    }

    @Test
    public void testCorrectFormatBrackets(){
        String phoneNumber = "+(45) 50 16 34";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correctly formatted number is formatted.", "tel:+(45)501634", formattedNumber);
    }
    @Test
    public void testCorrectFormatDot(){
        String phoneNumber = "0045. 50 16 34";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correctly formatted number is formatted.", "tel:0045.501634", formattedNumber);
    }
    @Test
    public void testCorrectFormatDash(){
        String phoneNumber = "0045- 50 16 34";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correctly formatted number is formatted.", "tel:0045-501634", formattedNumber);
    }

    @Test
    public void testCorrectFormatDashes(){
        String phoneNumber = "0045- 50- 16- 34";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Incorrectly formatted number is formatted.", "tel:0045-50-16-34", formattedNumber);
    }

        @Test
    public void testCorrectFormatZeros(){
        String phoneNumber = "0045 50 16 34";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correctly formatted number is formatted.", "tel:0045501634", formattedNumber);
    }

    @Test
    public void testNoNumber(){
        String phoneNumber = "tel:";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Incorrect number is formatted.","tel:", formattedNumber);
    }

    @Test
    public void testIncorrectChar(){
        String phoneNumber = "tel:+45501*634";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Correct number is not formatted.",phoneNumber, formattedNumber);
    }

    @Test
    public void testNoNumbers(){
        String phoneNumber = "tel:+wdwpkflwdelfcw!";
        String formattedNumber = FormatHelper.formatPhoneNumber(phoneNumber);
        assertEquals("Incorrect Number is formatted.","tel:+wdwpkflwdelfcw!", formattedNumber);
    }
}