/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.assembler;

import dk.stamkort.util.FSKException;
import dk.stamkort.util.FaultCode;
import org.junit.Test;

import java.util.Collections;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

public class SourceDataValidatorTest {

    @Test
    public void failIfNull() {
        try {
            SourceDataValidator.failIfNull(null);
            fail();
        } catch (FSKException e) {
            assertEquals(FaultCode.MAPPING_ERROR_MISSING_DATA, e.getFaultCode());
        }

        SourceDataValidator.failIfNull("hello");
    }

    @Test
    public void failIfEmpty() {
        try {
            SourceDataValidator.failIfEmpty(null);
            fail();
        } catch (FSKException e) {
            assertEquals(FaultCode.MAPPING_ERROR_MISSING_DATA, e.getFaultCode());
        }

        try {
            SourceDataValidator.failIfEmpty(Collections.emptyList());
            fail();
        } catch (FSKException e) {
            assertEquals(FaultCode.MAPPING_ERROR_MISSING_DATA, e.getFaultCode());
        }

        SourceDataValidator.failIfEmpty(Collections.singletonList("dummy"));
    }

    @Test
    public void failIfWrongSize() {
        try {
            SourceDataValidator.failIfWrongSize(null, 2, 2);
            fail();
        } catch (FSKException e) {
            assertEquals(FaultCode.MAPPING_ERROR_MISSING_DATA, e.getFaultCode());
        }

        try {
            SourceDataValidator.failIfWrongSize(Collections.singletonList("dummy"), 2, 2);
            fail();
        } catch (FSKException e) {
            assertEquals(FaultCode.MAPPING_ERROR_INVALID_DATA, e.getFaultCode());
        }

        SourceDataValidator.failIfWrongSize(Collections.singletonList("dummy"), 1, 2);
    }
}
